Package: scuttle
Type: Package
Authors@R: c(
        person("Aaron", "Lun", role = c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com"),
        person("Davis", "McCarthy", role="aut")
    )
Version: 1.22.0
Date: 2026-04-11
License: GPL-3
Title: Legacy Utilities for Single-Cell RNA-Seq Analysis
Description: 
    Provides some legacy utility functions for performing single-cell analyses.
    Most of these functions are deprecated in favor of newer, more performant alternatives.
    We just keep this package around for back-compatibility and to point to the replacement functions.
Depends: SingleCellExperiment
Imports: methods, utils, stats, Matrix, Rcpp, BiocGenerics, S4Vectors,
        BiocParallel, GenomicRanges, SummarizedExperiment, S4Arrays,
        MatrixGenerics, SparseArray, DelayedArray, beachmat
Suggests: BiocStyle, knitr, scRNAseq, rmarkdown, testthat,
        sparseMatrixStats, DelayedMatrixStats, scran
VignetteBuilder: knitr
biocViews: ImmunoOncology, SingleCell, RNASeq, QualityControl,
        Preprocessing, Normalization, Transcriptomics, GeneExpression,
        Sequencing, Software, DataImport
LinkingTo: Rcpp, beachmat, assorthead
SystemRequirements: C++17
RoxygenNote: 7.3.3
Encoding: UTF-8
NeedsCompilation: yes
git_url: https://git.bioconductor.org/packages/scuttle
git_branch: RELEASE_3_23
git_last_commit: d56a81e
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
Packaged: 2026-04-29 00:38:26 UTC; biocbuild
Author: Aaron Lun [aut, cre],
  Davis McCarthy [aut]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
