Scuttle 0.7.4
http://sourceforge.net/projects/scuttle/
http://scuttle.org/

Copyright (C) 2004 - 2008 Scuttle project
Available under the GNU General Public License

============
INSTALLATION
============

* Use the SQL contained in tables.sql to create the necessary database tables. This file was written specifically for MySQL, so may need rewritten if you intend to use a different database system.

* Edit config.inc.php.example and save the changes as a new config.inc.php file in the same directory.

* Set the CHMOD permissions on the /cache/ subdirectory to 777