# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

aggregate_across_cells <- function(x, groups, num_groups, do_sum, do_detected, do_median, num_threads) {
    .Call(`_scuttle_aggregate_across_cells`, x, groups, num_groups, do_sum, do_detected, do_median, num_threads)
}

cumulative_prop <- function(input, top) {
    .Call(`_scuttle_cumulative_prop`, input, top)
}

downsample_global <- function(input, prop_global, num_threads, already_integer, output_sparse, output_integer) {
    .Call(`_scuttle_downsample_global`, input, prop_global, num_threads, already_integer, output_sparse, output_integer)
}

downsample_column <- function(input, prop_col, already_integer, output_sparse, output_integer) {
    .Call(`_scuttle_downsample_column`, input, prop_col, already_integer, output_sparse, output_integer)
}

fit_linear_model <- function(qr, qraux, exprs, get_coefs, nthreads) {
    .Call(`_scuttle_fit_linear_model`, qr, qraux, exprs, get_coefs, nthreads)
}

pool_size_factors <- function(exprs, pseudo_cell, order, pool_sizes) {
    .Call(`_scuttle_pool_size_factors`, exprs, pseudo_cell, order, pool_sizes)
}

set_executor <- function(ptr) {
    .Call(`_scuttle_set_executor`, ptr)
}

sum_row_counts <- function(counts, genes, runs) {
    .Call(`_scuttle_sum_row_counts`, counts, genes, runs)
}

